package com.bril.webfile;

import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Toast;

import com.bril.webfile.net.client.FileClient;
import com.bril.webfile.response.ChildrenBeans;
import com.bril.webfile.response.NextCloudResponse;
import com.bril.webfile.treeview.adapter.SimpleTreeListViewAdapter;
import com.bril.webfile.treeview.adapter.TreeViewAdapter;
import com.bril.webfile.treeview.node.Node;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Created by Xinghx on 2018/6/26 0026.
 */

public class NextCloudActivity extends AppCompatActivity {
    ListView mTree;
    List<ChildrenBeans> childrenBeansList = new ArrayList<>();
    private String userId;
    private static final String TAG = "NextCloudActivity";
    private String moduleName;
    private LinearLayout file_img_backs;

    private NextCloudResponse cloudResponse;
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_file_layout);
        userId = getIntent().getStringExtra("userId");
        moduleName = getIntent().getStringExtra("moduleName");
        mTree = (ListView) findViewById(R.id.file_lv);
        file_img_backs = (LinearLayout) findViewById(R.id.file_img_backs);

        file_img_backs.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
        initData();
    }

    public void initData() {

        String realType = "";
        switch (moduleName) {
            case "myselfNetDisk":
                realType = "1";
                break;
            case "shareFolder":
                realType = "2";

                break;
            case "documentManager":
                realType = "3";
                break;
        }

        FileClient fileClient = new FileClient();
        fileClient.nextCloud(userId, realType).enqueue(new Callback<NextCloudResponse>() {
            @Override
            public void onResponse(Call<NextCloudResponse> call, Response<NextCloudResponse> response) {
                try {
                    cloudResponse = response.body();

                    List<NextCloudResponse.ListBean> list = response.body().getList();
                    for (NextCloudResponse.ListBean listBean : list) {
                        ChildrenBeans childrenBeans = new ChildrenBeans();
                        String type = listBean.getType();
                        String name = listBean.getName();
                        childrenBeans.setName(name);
                        childrenBeans.setType(type);
                        childrenBeansList.add(childrenBeans);
                        addData(listBean.getChildren());
                    }
                    for (ChildrenBeans childrenBeans : childrenBeansList) {
                        Log.e(TAG, childrenBeans.toString() );
                    }

                    SimpleTreeListViewAdapter simpleTreeListViewAdapter
                            = new SimpleTreeListViewAdapter(mTree, NextCloudActivity.this, childrenBeansList, 0);
                    mTree.setAdapter(simpleTreeListViewAdapter);
                    simpleTreeListViewAdapter.setOnNodeClickListener(new TreeViewAdapter.OnTreeNodeClickListener() {
                        @Override
                        public void onClick(Node node, int position) {
                            if (node.isFile()) {
                                Toast.makeText(NextCloudActivity.this, node.getName(), Toast.LENGTH_SHORT).show();
                                Intent intent = new Intent();
                                intent.putExtra("absolutePath",cloudResponse.getAbsolutePath());
                                intent.putExtra("filePaths", node.getName());
                                setResult(0x013, intent);
                                finish();
                            }
                        }
                    });
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }

            }

            @Override
            public void onFailure(Call<NextCloudResponse> call, Throwable t) {
                t.printStackTrace();
            }
        });
    }

    private void addData(List<NextCloudResponse.ListBean.ChildrenBean> mList) {

        for (NextCloudResponse.ListBean.ChildrenBean childrenBean : mList) {
            if (childrenBean == null) return;
            ChildrenBeans childrenBeans = new ChildrenBeans();
            String name = childrenBean.getName();
            String type = childrenBean.getType();
            childrenBeans.setName(name);
            childrenBeans.setType(type);
            childrenBeansList.add(childrenBeans);
            if (childrenBean.getChildrenBeen() != null) {
                addData(childrenBean.getChildrenBeen());
            }
        }
    }
}
