package com.bril.webfile.treeview.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.ListView;

import com.bril.webfile.treeview.helper.TreeHelper;
import com.bril.webfile.treeview.node.Node;

import java.util.List;

/**
 * Created by Xinghx on 2018/6/25 0025.
 */

public abstract class TreeViewAdapter<T> extends BaseAdapter{

    private Context mContext;
    private List<Node> mAllNodes;
    private List<Node> mVisiableNodes;
    protected LayoutInflater mInflater;
    private ListView tree;
    public interface OnTreeNodeClickListener{
        void onClick(Node node,int position);
    }

    private OnTreeNodeClickListener mListener;

    public void setOnNodeClickListener(OnTreeNodeClickListener mListener) {
        this.mListener = mListener;
    }

    public TreeViewAdapter(ListView tree, Context context, List<T> datas, int defaultLevel) throws IllegalAccessException {
        this.mContext = context;
        mAllNodes = TreeHelper.getSortedNodes(datas, defaultLevel);
        mVisiableNodes = TreeHelper.filterVisibleNodes(mAllNodes);
        mInflater= LayoutInflater.from(context);
        this.tree = tree;

        tree.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                expandOrCollapse(position);
                if (mListener != null) {
                    mListener.onClick(mVisiableNodes.get(position),position);
                }
            }
        });
    }

    /**
     * 点击收缩或者展开
     * @param position
     */
    private void expandOrCollapse(int position) {
        Node n = mVisiableNodes.get(position);
        if (n.isFile()) return;
        n.setExpand(!n.isExpand());
        mVisiableNodes = TreeHelper.filterVisibleNodes(mAllNodes);
        notifyDataSetChanged();
    }

    @Override
    public int getCount() {
        return mVisiableNodes.size();
    }

    @Override
    public Object getItem(int position) {
        return mVisiableNodes.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        Node node = mVisiableNodes.get(position);
        convertView = getConvertView(node, position, convertView, parent);
        // 设置内边距
        convertView.setPadding(node.getLevel() * 50 ,3 , 3 ,3);
        return convertView;
    }

    public abstract View getConvertView(Node node,int position, View convertView, ViewGroup parent);
}
