package com.bril.webfile.treeview.helper;

import android.util.Log;

import com.bril.webfile.R;
import com.bril.webfile.treeview.annotation.NodeName;
import com.bril.webfile.treeview.annotation.NodeType;
import com.bril.webfile.treeview.node.Node;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Xinghx on 2018/6/25 0025.
 */

public class TreeHelper {
    private static final String TAG = "TreeHelper";
    /**
     * 把用户提供的数据 转为NODE
     *
     * @param <T>
     * @return
     */
    public static <T> List<Node> convertData2Nodes(List<T> datas) throws IllegalAccessException {
        List<Node> nodes = new ArrayList<>();
        Node node = null;

        for (T t : datas) {
            String type = "folder";
            String name = "";

            Class clazz = t.getClass();
            Field[] fields = clazz.getDeclaredFields();
            for (Field field : fields) {
                if (field.getAnnotation(NodeType.class) != null) {
                    field.setAccessible(true);
                    type = (String) field.get(t);
                }
                if (field.getAnnotation(NodeName.class) != null) {
                    field.setAccessible(true);
                    name = (String) field.get(t);
                }
            }

            node = new Node(name, type);
            nodes.add(node);
        }

        // 设置node间节点关系
        for (int i = 0; i < nodes.size(); i++) {
            Node n = nodes.get(i);
            for (int j = i + 1; j < nodes.size(); j++) {
                Node m = nodes.get(j);
                Log.d(TAG, "" + m.getName() + "  " + n.getName());
                String aaa = m.getName();
                String bbb = n.getName();
                if (aaa.lastIndexOf("/") != -1) {
                    aaa = aaa.substring(0, aaa.lastIndexOf("/"));
                }

                if (aaa.equals(n.getName()) && n.isFolder()) {
                    Log.d(TAG, "convertData2Nodes: " + m.getName() + " " + n.getName());
                    n.getChildren().add(m);
                    m.setParent(n);
                } else if (n.getName().equals(aaa) && m.isFolder()) {
                    Log.d(TAG, "convertData2Nodes else : " + m.getName() + " " + n.getName());
                    m.getChildren().add(n);
                    n.setParent(m);
                }
            }
        }

        for (Node n : nodes) {
            setNodeIcons(n);
        }


        //方法2
//        for (int i = 0; i < datas.size(); i++) {
//            Node n = nodes.get(i);
//            for (int j = i + 1; j < nodes.size(); j++) {
//                Node m = nodes.get(j);
//                if (m.getpId() instanceof String) {
//                    if (m.getpId().equals(n.getId())) {
//                        n.getChildren().add(m);
//                        m.setParent(n);
//                    } else if (m.getId().equals(n.getpId())) {
//                        m.getChildren().add(n);
//                        n.setParent(m);
//                    }
//                } else {
//                    if (m.getpId() == n.getId()) {
//                        n.getChildren().add(m);
//                        m.setParent(n);
//                    } else if (m.getId() == n.getpId()) {
//                        m.getChildren().add(n);
//                        n.setParent(m);
//                    }
//                }
//            }
        return nodes;
    }

    /**
     * 为node设置图标
     *
     * @param n
     */
    public static void setNodeIcons(Node n) {
        if (n.getChildren().size() > 0 && n.isExpand() && n.isFolder()) {
            n.setIcon(R.drawable.tree_ex);

        } else if (n.getChildren().size() > 0 && !n.isExpand() && n.isFolder()) {
            n.setIcon(R.drawable.tree_ec);
        } else {
            n.setIcon(-1);
        }
    }

    public static <T> List<Node> getSortedNodes(List<T> datas,int defaultExpandLevel) throws IllegalAccessException {
        List<Node> result = new ArrayList<>();
        List<Node> nodes = convertData2Nodes(datas);

        List<Node> rootNodes = getRootNodes(nodes);

        for (Node node : rootNodes) {
            addNode(result, node, defaultExpandLevel, 1);
        }

        Log.e("TAG", result.size() + "");
        return result;

    }


    /**
     * 从所有节点中过滤出根节点
     *
     * @param nodes
     * @return
     */
    private static List<Node> getRootNodes(List<Node> nodes) {
        List<Node> root = new ArrayList<Node>();
        for (Node node : nodes) {
            if (node.isRoot()) {
                root.add(node);
            }
        }
        return root;
    }

    private static void addNode(List<Node> result, Node node,
                                int defaultExpandLevel, int currentLevel) {
        result.add(node);
        if (defaultExpandLevel >= currentLevel) {
            node.setExpand(true);
        }
        if (node.isFile())
            return;

        for (int i = 0; i < node.getChildren().size(); i++) {
            addNode(result, node.getChildren().get(i), defaultExpandLevel,
                    currentLevel + 1);
        }

    }


    /**
     * 过滤出可见的节点
     *
     * @param nodes
     * @return
     */
    public static List<Node> filterVisibleNodes(List<Node> nodes) {
        List<Node> result = new ArrayList<Node>();

        for (Node node : nodes) {
            if (node.isRoot() || node.isParentExpand()) {
                setNodeIcons(node);
                result.add(node);
            }
        }
        return result;
    }
}
